#include "gtweakui.h"

GSList *gt_add(GSList *prefs, GSList *item) {
	prefs = g_slist_prepend(prefs, item);
	return prefs;
}

GSList *gt_pair(GConfClient *gcid, char *gconf, GtkWidget *widget, gpointer init, char *event, gpointer gtk_callback, gpointer gconf_callback, gpointer kill_callback) {
	GSList *pair = NULL;

	pair = g_slist_prepend(pair, gcid);
	pair = g_slist_prepend(pair, init);
	pair = g_slist_prepend(pair, widget);
	pair = g_slist_prepend(pair, gconf);
	pair = g_slist_prepend(pair, kill_callback);

	gsc(widget, event, gtk_callback, pair);
	gconfc(gcid, gconf, gconf_callback, pair);

	return pair;
}

void gt_callback_gconf_bool(GConfClient *client, guint cid, GConfEntry *e, gpointer d) {
	GSList *type = (GSList *)d, *gconf = type->next, *toggle = gconf->next, *old = toggle->next;

	if ((gconf_entry_get_value(e) != NULL) && (gconf_entry_get_value(e)->type == GCONF_VALUE_BOOL)) {
		gboolean new = gconf_value_get_bool(gconf_entry_get_value(e));
		if ((gboolean)(old->data) != new) {
			old->data = (gpointer)new;
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON((GtkWidget *)(toggle->data)), new);
		}
	}
}

void gt_callback_checkbox_bool(GtkWidget *widget, gpointer d) {
	GSList *type = (GSList *)d, *gconf = type->next, *toggle = gconf->next, *old = toggle->next, *gc = old->next;
	gboolean new = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));

	if (new != (gboolean)(old->data)) {
		old->data = (gpointer)new;
		gconf_client_set_bool(gc->data, gconf->data, new, NULL);
	}
}

GSList *gt_bool(GConfClient *gcid, char *gconf, GtkWidget *checkbox) {
	gboolean init = gconf_client_get_bool(gcid, gconf, NULL);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox), init);
	return gt_pair(gcid, gconf, checkbox, (gpointer)init, "toggled", gt_callback_checkbox_bool, gt_callback_gconf_bool, NULL);
}

void gt_callback_gconf_string(GConfClient *client, guint cid, GConfEntry *e, gpointer d) {
	GSList *type = (GSList *)d, *gconf = type->next, *entry = gconf->next, *old = entry->next, *gc = old->next;

	if ((gconf_entry_get_value(e) != NULL) && (gconf_entry_get_value(e)->type == GCONF_VALUE_STRING)) {
		const gchar *new = gconf_value_get_string(gconf_entry_get_value(e));
		if (g_utf8_collate(((GString *)(old->data))->str, new) != 0) {
			old->data = g_string_assign((GString *)(old->data), new);
			gtk_entry_set_text(GTK_ENTRY(entry->data), new);
		}
	}
}

void gt_callback_entry_string(GtkWidget *w, gpointer d) {
	GSList *type = (GSList *)d, *gconf = type->next, *entry = gconf->next, *old = entry->next, *gc = old->next;

	const gchar *new = gtk_entry_get_text(GTK_ENTRY(w));

	if (g_utf8_collate(((GString *)(old->data))->str, new) != 0) {
		old->data = g_string_assign((GString *)(old->data), new);
		gconf_client_set_string(gc->data, gconf->data, new, NULL);
	}
}

void gt_callback_kill_string(GSList *pair) {
	g_string_free((GString *)pair->next->next->next->data, TRUE);
}

GSList *gt_string(GConfClient *gcid, char *gconf, GtkWidget *entry) {
	gchar *cinit = gconf_client_get_string(gcid, gconf, NULL);
	GString *init = g_string_new("");
	if (cinit != NULL) {
		init = g_string_assign(init, cinit);
		gtk_entry_set_text(GTK_ENTRY(entry), cinit);
		g_free(cinit);
	}
	return gt_pair(gcid, gconf, entry, (gpointer)init, "changed", gt_callback_entry_string, gt_callback_gconf_string, gt_callback_kill_string);
}

void gt_free(GSList *prefs) {
	GSList *head = prefs, *old;
	void (*freer)(GSList *);

	while (head != NULL) {
		freer = ((GSList *)(head->data))->data;
		if (freer != NULL)
			freer(head->data);
		g_slist_free(head->data);
		head = head->next;
	}

	g_slist_free(prefs);
}
