#include "gtweakui.h"

GladeXML *ws;
static gboolean oldcheck[3];
static gboolean icheck[3];
GConfClient* gc;

void doquit() {
	GSList *infos = NULL;
	int x;
	char *messages[3] = {N_("Detachable menubars"), N_("Menus can be torn-off"), N_("Can change menu accelerators")};

	for (x = 0; x < 3; x ++)
		if (icheck[x] != oldcheck[x])
			infos = g_slist_prepend(infos, messages[x]);
	
	if (infos != NULL) {
		gtk_widget_hide(xmlg(ws, "menus"));
		notice(infos);
		g_slist_free(infos);
	} else {
		gtk_main_quit();
	}
}

void on_close_clicked(GtkWidget *widget, gpointer data) {
	doquit();
}

gboolean delete_event(GtkWidget *widget, GdkEvent *event, gpointer data) {
	doquit();
	return TRUE;
}

void updatePreview() {
	static GtkWidget *t1,*t2 = NULL, *pr;
	static int oldimage = -1;
	int image = 0;
	char imagename[]="menusx.png";

	if (t2 == NULL) {
		t1 = glade_xml_get_widget(ws, "detachable_menubar");
		t2 = glade_xml_get_widget(ws, "tearoff_menus");
		pr = glade_xml_get_widget(ws, "preview");
	}
	
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(t1)))
		image += 1;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(t2)))
		image += 2;

	if (image != oldimage) {
		oldimage = image;
		imagename[5] = '0' + image;
		gtk_image_set_from_file(GTK_IMAGE(pr), gnome_program_locate_file(gtweakui, GNOME_FILE_DOMAIN_APP_DATADIR, imagename, FALSE, NULL));
	}
}

static void gconfChangedCallback (GConfClient* client, guint cnxn_id, GConfEntry* entry, gpointer data) {
	char *str = (char *)data;
	static GtkWidget *t[3]={NULL,NULL,NULL};
	int index;

	index = str[0] - '1';
	
	if (t[0] == NULL) {
		t[0] = glade_xml_get_widget(ws, "detachable_menubar");
		t[1] = glade_xml_get_widget(ws, "tearoff_menus");
		t[2] = glade_xml_get_widget(ws, "change_accels");
	}
	
	if ((gconf_entry_get_value (entry) != NULL) &&
			gconf_entry_get_value (entry)->type == GCONF_VALUE_BOOL) {
		gboolean new = gconf_value_get_bool(gconf_entry_get_value(entry));
		if (oldcheck[index] != new) {
			oldcheck[index] = new;
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t[index]), new);
			if ((str[0] == '1') || (str[0] == '2'))
				updatePreview();
		}
	}
}

static void checkboxCallback(GtkWidget *widget, gpointer data) {
	char *str = (char *)data;
	static char *keys[3] = {"/desktop/gnome/interface/menubar_detachable", "/desktop/gnome/interface/menus_have_tearoff", "/desktop/gnome/interface/can_change_accels"};
	int index = str[0] - '1';
	gboolean state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));

	if (state != oldcheck[index]) {
		oldcheck[index] = state;
		gconf_client_set_bool(gc, keys[index], state, NULL);
		if ((str[0] == '1') || (str[0] == '2'))
			updatePreview();
	}
}

void hookupcallbacks() {
	GtkWidget *w;

	gc = gconf_client_get_default();
	gconf_client_add_dir(gc, "/desktop/gnome/interface", GCONF_CLIENT_PRELOAD_NONE, NULL);

	w = glade_xml_get_widget(ws, "close_button");
	g_signal_connect(G_OBJECT(w), "clicked", G_CALLBACK(on_close_clicked), NULL);
	
	w = glade_xml_get_widget(ws, "about_button");
	g_signal_connect(G_OBJECT(w), "clicked", G_CALLBACK(about), glade_xml_get_widget(ws, "menus"));
	
	w = glade_xml_get_widget(ws, "menus");
	g_signal_connect(G_OBJECT(w), "delete_event", G_CALLBACK(delete_event), NULL);

	w = glade_xml_get_widget(ws, "detachable_menubar");
	g_signal_connect(G_OBJECT(w), "toggled", G_CALLBACK(checkboxCallback), "1");
	gconf_client_notify_add(gc, "/desktop/gnome/interface/menubar_detachable", gconfChangedCallback, "1", NULL, NULL);
	icheck[0] = oldcheck[0] = gconf_client_get_bool (gc, "/desktop/gnome/interface/menubar_detachable", NULL);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), oldcheck[0]);

	w = glade_xml_get_widget(ws, "tearoff_menus");
	g_signal_connect(G_OBJECT(w), "toggled", G_CALLBACK(checkboxCallback), "2");
	gconf_client_notify_add(gc, "/desktop/gnome/interface/menus_have_tearoff", gconfChangedCallback, "2", NULL, NULL);
	icheck[1] = oldcheck[1] = gconf_client_get_bool (gc, "/desktop/gnome/interface/menus_have_tearoff", NULL);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), oldcheck[1]);
	
	w = glade_xml_get_widget(ws, "change_accels");
	g_signal_connect(G_OBJECT(w), "toggled", G_CALLBACK(checkboxCallback), "3");
	gconf_client_notify_add(gc, "/desktop/gnome/interface/can_change_accels", gconfChangedCallback, "3", NULL, NULL);
	icheck[2] = oldcheck[2] = gconf_client_get_bool (gc, "/desktop/gnome/interface/can_change_accels", NULL);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), oldcheck[2]);

	gtk_widget_show_all(xmlg(ws, "menus"));
}

int main(int argc, char *argv[]) {
	gtweakui = gnome_program_init(PACKAGE_NAME, PACKAGE_VERSION, LIBGNOMEUI_MODULE, argc, argv, GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR, NULL);
	gconf_init(argc, argv, NULL);
	
	bindtextdomain (GETTEXT_PACKAGE, GTWEAKUILOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);

	ws = glade_xml_new(gnome_program_locate_file(gtweakui, GNOME_FILE_DOMAIN_APP_DATADIR, "menus.glade", FALSE, NULL), NULL, NULL);

	hookupcallbacks();
	updatePreview();

	gtk_main();

	return 0;
}
